/*
 * $XConsortium: x386Init.c,v 1.2 91/08/20 15:39:58 gildea Exp $
 *
 * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Thomas Roell not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Thomas Roell makes no representations
 * about the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * THOMAS ROELL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THOMAS ROELL BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "X.h"
#include "Xmd.h"
#include "input.h"
#include "mipointer.h"
#include "servermd.h"
#include "scrnintstr.h"
#include "site.h"

#include "compiler.h"

#include "x386Procs.h"
#include "x386OSD.h"
#include "vga.h"


#ifdef XTESTEXT1
#include "atKeynames.h"
extern int xtest_command_key;
#endif /* XTESTEXT1 */

#ifdef linux

/*
 * the fontserver only works with tcp and won't compile. the server depends
 * on having this function, so we oblige them
 */
fs_register_fpe_functions()
{
	return 0;
}

#endif /* linux */

x386InfoRec x386Info;
int         x386ScreenIndex;
extern char *defaultFontPath;
extern char *rgbPath;
static int Graphics_mode = G640x480x16;

static struct { char* name; int mode; } 
ega_vga_modes[] = {
  { "320x200x16",  G320x200x16 },
  { "640x200x16",  G640x200x16 },
  { "640x350x16",  G640x350x16 },
  { "640x480x16",  G640x480x16 },
  { "320x200x256", G320x200x256},
  { "320x240x256", G320x240x256},
  { "320x400x256", G320x400x256},
  { "360x480x256", G360x480x256},
  { 0,0 }};

extern char* getenv();

void x386Config()
{
  x386Info.dontZap       = FALSE;
  x386Info.serverNumLock = FALSE;
  x386Info.xleds         = 0L;
  x386Info.kbdDelay      = 500;
  x386Info.kbdRate       = 30;

  x386Info.baudRate        = 1200;
  x386Info.sampleRate      = 0;
  x386Info.emulate3Buttons = TRUE;

  x386Info.kbdProc    = x386KbdProc;
  x386Info.kbdEvents  = x386KbdEvents;
  x386Info.mseProc    = x386MseProc;
  x386Info.mseEvents  = x386MseEvents;
  x386Info.mseType    = 0;
  x386Info.mseDevice  = "/dev/mouse";

  if (!(defaultFontPath=getenv("FONT_PATH")) )
    defaultFontPath = "/usr/lib/X11/fonts/misc/";

  if (!(rgbPath=getenv("RGB_PATH")))
    rgbPath = "/usr/lib/X11/rgb";
  
  x386Info.consoleFd = vga_console();
}

/*
 * InitOutput --
 *	Initialize screenInfo for all actually accessible framebuffers.
 *      That includes vt-manager setup, querying all possible devices and
 *      collecting the pixmap formats.
 */
static int ocon = -1;

static ScreenRec X386Screen;


extern miPointerScreenFuncRec x386PointerScreenFuncs;
int _DEBUG=0;

Bool VGAInit(i,pScreen,argc,argv)
     int i;
     ScreenPtr	pScreen;
     int     	argc;
     char    	**argv;
{
  extern Bool vga_SaveScreen();

  int r = 255,g=255,b=255;
  char* white;

  if(!mfbScreenInit(pScreen, vga_base(),640,480,75,75,640)){
    ErrorF("Error initializing DRAGON\n");
    return FALSE;
  }
  pScreen->whitePixel = 1;
  pScreen->blackPixel = 0;
  pScreen->SaveScreen = vga_SaveScreen;

  miDCInitialize (pScreen, &x386PointerScreenFuncs);
  mfbCreateDefColormap(pScreen);
  if(!_DEBUG) {
    vga_setmode(Graphics_mode);
    vga_mfbinit();
    if (white=(char *)getenv("WHITE"))
      sscanf(white,"%d%d%d",&r,&g,&b);
    vga_setpalette(1,r,g,b);      
  }

  return TRUE;
}
       
void
InitOutput(pScreenInfo, argc, argv)
     ScreenInfo	*pScreenInfo;
     int     	argc;
     char    	**argv;
{
  int                    i, j, index, fd;
  static int             numFormats = 0;
  static PixmapFormatRec formats[MAXFORMATS];
  static unsigned long   generation = 0;
  char *modename = getenv("DISPLAY_MODE");
   
  if (serverGeneration == 1) {

    ErrorF("X386 Version 1.2 / X Windows System [linux v1.1]\n");
    ErrorF("(protocol Version %d, revision %d, vendor release %d)\n\n",
	   X_PROTOCOL, X_PROTOCOL_REVISION, VENDOR_RELEASE );
    ErrorF("VGA/EGA port by marc.hoffman@analog.com\n\n");


  if (modename) {
    int i;
    for (i=0;ega_vga_modes[i].name!=0;i++)
      if (strcmp(modename,ega_vga_modes[i].name)==0) {
	Graphics_mode = ega_vga_modes[i].mode;
	break;
      }
  }

    vga_setmode(Graphics_mode);
    vga_setmode(TEXT);

    x386Config();

    /*
     * collect all possible formats
     */
    formats[0].depth = 1;
    formats[0].bitsPerPixel = 1;
    formats[0].scanlinePad = BITMAP_SCANLINE_PAD;
    
    formats[1].depth = 4;
    formats[1].bitsPerPixel = 4;
    formats[1].scanlinePad =  BITMAP_SCANLINE_PAD;
    numFormats = 1;
  }
  pScreenInfo->screens[0] = &X386Screen;

  /*
   * Use the previous collected parts to setup pScreenInfo
   */
#ifdef MFB_VGA
  pScreenInfo->imageByteOrder     = IMAGE_BYTE_ORDER;
#else
  pScreenInfo->imageByteOrder     = MSBFirst; /* IMAGE_BYTE_ORDER; works...*/
#endif
  pScreenInfo->bitmapScanlineUnit = BITMAP_SCANLINE_UNIT;
  pScreenInfo->bitmapScanlinePad  = BITMAP_SCANLINE_PAD;
  pScreenInfo->bitmapBitOrder     =  BITMAP_BIT_ORDER;
  pScreenInfo->numPixmapFormats   = numFormats;

  for ( i=0; i < numFormats; i++ ) pScreenInfo->formats[i] = formats[i];

  if (generation != serverGeneration)
    {
      x386ScreenIndex = AllocateScreenPrivateIndex();
      generation = serverGeneration;
    }
  index = AddScreen(VGAInit, argc, argv);

  RegisterBlockAndWakeupHandlers(x386Block, x386Wakeup, (void *)0);
}



/*
 * InitInput --
 *      Initialize all supported input devices...what else is there
 *      besides pointer and keyboard? Two DeviceRec's are allocated and
 *      registered as the system pointer and keyboard devices.
 */

void
InitInput(argc, argv)
     int     	  argc;
     char    	  **argv;
{
  x386Info.vtRequestsPending = FALSE;
  x386Info.inputPending = FALSE;
#ifdef XTESTEXT1
  xtest_command_key = KEY_SysReqest + MIN_KEYCODE;
#endif /* XTESTEXT1 */

  x386Info.pKeyboard = AddInputDevice(x386Info.kbdProc, TRUE); 
  x386Info.pPointer =  AddInputDevice(x386Info.mseProc, TRUE);
  RegisterKeyboardDevice(x386Info.pKeyboard); 
  RegisterPointerDevice(x386Info.pPointer); 

  miRegisterPointerDevice(screenInfo.screens[0], x386Info.pPointer);
  mieqInit (x386Info.pKeyboard, x386Info.pPointer);
}



/*
 * ddxGiveUp --
 *      Device dependent cleanup. Called by by dix before normal server death.
 *      For SYSV386 we must switch the terminal back to normal mode. No error-
 *      checking here, since there should be restored as much as possible.
 */

void
ddxGiveUp()
{
  vga_setmode(TEXT);
}



/*
 * AbortDDX --
 *      DDX - specific abort routine.  Called by AbortServer(). The attempt is
 *      made to restore all original setting of the displays. Also all devices
 *      are closed.
 */

void
AbortDDX()
{
  int i;

  /*
   * try to deinitialize all input devices
   */
  if (x386Info.pPointer) (x386Info.mseProc)(x386Info.pPointer, DEVICE_CLOSE);
  if (x386Info.pKeyboard) (x386Info.kbdProc)(x386Info.pKeyboard, DEVICE_CLOSE);

  /*
   * This is needed for a abnormal server exit, since the normal exit stuff
   * MUST also be performed (i.e. the vt must be left in a defined state)
   */
  ddxGiveUp();
}



/*
 * ddxProcessArgument --
 *	Process device-dependent command line args. Returns 0 if argument is
 *      not device dependent, otherwise Count of number of elements of argv
 *      that are part of a device dependent commandline option.
 */

/* ARGSUSED */
int
ddxProcessArgument (argc, argv, i)
     int argc;
     char *argv[];
     int i;
{
  return 0;
}


/*
 * ddxUseMsg --
 *	Print out correct use of device dependent commandline options.
 *      Maybe the user now knows what really to do ...
 */

void
ddxUseMsg()
{
  ErrorF("\n");
  ErrorF("\n");
  ErrorF("Device Dependent Usage\n");
  ErrorF("\n");
}
